==> hyperdiv-0.0.2-py3-none-any.whl: hyperdiv/core/tests/context_tests.py <==
import time
import threading
from nose.tools import assert_raises
import hyperdiv as hd
from .. import context
from ..unittests import simple_test
from .utils import MockConnection


def test_event_bus():
    results = []

    def listener1(*args, **kwargs):
        results.append(("listener1", args, kwargs))

    def listener2(*args, **kwargs):
        results.append(("listener2", args, kwargs))

    bus = context.EventBus()

    bus.on("foo", listener1)
    bus.on("foo", listener2)
    bus.on("bar", listener1)

    assert bus.listeners == dict(foo=[listener1, listener2], bar=[listener1])

    bus.dispatch("foo", 1, x=2)
    bus.dispatch("bar", 2, 3, y=3, z=4)

    assert results == [
        ("listener1", (1,), {"x": 2}),
        ("listener2", (1,), {"x": 2}),
        ("listener1", (2, 3), {"y": 3, "z": 4}),
    ]

    bus.off("foo", listener1)

    assert bus.listeners == dict(foo=[listener2], bar=[listener1])

    bus.off("foo")

    assert bus.listeners == dict(bar=[listener1])

    bus.off("bar", listener1)

    assert bus.listeners == dict()


def test_collector_stack():
    s = context.CollectorStack()

    collector1 = []
    s.push(collector1)
    s.collect("Bunnies")
    s.collect("Bears")
    assert s.stack == [["Bunnies", "Bears"]] == [collector1]
    assert s.current() == collector1

    collector2 = []
    s.push(collector2)
    s.collect("Apples")
    s.collect("Oranges")
    assert (
        s.stack
        == [["Bunnies", "Bears"], ["Apples", "Oranges"]]
        == [collector1, collector2]
    )

    s.paused = True
    s.collect("Hello")
    assert s.stack == [collector1, collector2]
    s.paused = False

    assert s.pop() == collector2
    assert s.stack == [collector1]
    assert s.pop() == collector1
    assert s.stack == []

    assert_raises(Exception, s.current)


@simple_test
def test_paused_collector(ctx):
    with context.paused_collector():
        ctx.collector_stack.collect("Bunnies")
        ctx.collector_stack.collect("Bears")
    assert ctx.collector_stack.stack == [[]]


def test_scope_stack():
    s = context.ScopeStack()
    s.push("Bunnies")
    s.push("Bears")
    s.push("Apples")
    assert s.stack == ["Bunnies", "Bears", "Apples"]
    assert str(s) == "Bunnies:Bears:Apples"
    s.pop()
    assert s.stack == ["Bunnies", "Bears"]
    assert str(s) == "Bunnies:Bears"
    s.pop()
    s.pop()
    assert s.stack == []
    assert str(s) == ""


def test_state_update_batch():
    b = context.StateUpdateBatch()
    assert b.state == dict()

    b.set_state("a", "b", "value1")
    b.set_state("c", "d", "value2")

    assert b.has_state("a", "b")
    assert b.has_state("c", "d")
    assert not b.has_state("x", "y")

    assert_raises(Exception, b.set_state, "a", "b", "value")

    assert b.state == {("a", "b"): "value1", ("c", "d"): "value2"}


def test_pending_batches():
    evt = threading.Event()
    pb = context.PendingBatches(evt)
    pb.set_state_bulk(
        [
            ("a", "b", "value1"),
            ("c", "d", "value2"),
            ("x", "y", "value3"),
        ]
    )
    pb.set_state_bulk([("a", "b", "value4")])
    pb.set_state_bulk([("foo", "bar", "value5")])
    pb.set_state_bulk([("a", "b", "value6")])
    pb.set_state_bulk([("bam", "baz", "value7")])
    pb.set_state_bulk([("foo", "bar", "value8")])

    # Does nothing
    pb.set_state_bulk([])

    # Can't repeat keys in a batch
    assert_raises(
        Exception,
        pb.set_state_bulk,
        [("a", "b", "value9"), ("c", "d", "value10"), ("a", "b", "value11")],
    )

    assert len(pb.pending_batches) == 3
    [batch1, batch2, batch3] = pb.pending_batches

    assert batch1.state == {
        ("a", "b"): "value1",
        ("c", "d"): "value2",
        ("x", "y"): "value3",
        ("foo", "bar"): "value5",
        ("bam", "baz"): "value7",
    }
    assert batch2.state == {
        ("a", "b"): "value4",
        ("foo", "bar"): "value8",
    }
    assert batch3.state == {
        ("a", "b"): "value6",
    }

    assert pb.dequeue_batch() == batch1
    assert pb.dequeue_batch() == batch2
    assert pb.dequeue_batch() == batch3
    assert pb.dequeue_batch() is None

    assert pb.pending_batches == []


def test_context():
    button = None
    text = None

    def function():
        nonlocal button, text
        button = hd.button("Hello")
        text = hd.text("Hello")

    messages = []
    conn = MockConnection(messages)
    c = context.Context(function)
    context.Context.set_current_context(c)
    c.start(conn)

    conn.has_message.wait()

    assert len(messages) == 1
    msg = messages[0]

    assert "listeners" not in msg
    assert "children" in msg["dom"]  # root container

    # head and location singletons are first
    assert msg["dom"]["children"][0]["name"] == "head"
    assert msg["dom"]["children"][1]["name"] == "location"
    # our button and text are next
    assert msg["dom"]["children"][2] == button.to_dom()
    assert msg["dom"]["children"][3] == text.to_dom()
    context.Context.clear_current_context()
    c.stop()


def test_context_update():
    button = None
    text = None

    def function():
        nonlocal button, text
        button = hd.button("Hello")
        state = hd.state(shown=False)
        if state.shown:
            text = hd.text("Hello")
        if button.clicked:
            state.shown = True

    messages = []
    conn = MockConnection(messages)
    c = context.Context(function)
    context.Context.set_current_context(c)
    c.start(conn)

    conn.has_message.wait()

    assert len(messages) == 1
    msg = messages[0]

    assert msg["listeners"] == dict(add=[(button.key, "clicked")])
    assert "children" in msg["dom"]  # root container

    assert len(msg["dom"]["children"]) == 3
    assert msg["dom"]["children"][0]["name"] == "head"
    assert msg["dom"]["children"][1]["name"] == "location"
    # Text is not yet visible, only the button
    assert msg["dom"]["children"][2] == button.to_dom()

    # Simulate click:

    c.set_state(button.key, "clicked", True)
    # Sleep a bit to increase the chances that the update is applied.
    time.sleep(0.1)

    # One more message was sent:
    assert len(messages) == 2
    msg = messages[1]

    assert "diff" in msg
    assert "dom" not in msg

    # The message is a diff inserting the text element after the button, at index 2
    assert msg["diff"] == [
        ("subdiff_list", "children", [("insert", 3, [text.to_dom()])])
    ]
    context.Context.clear_current_context()
    c.stop()


def test_context_refresh():
    numruns = 0

    def function():
        nonlocal numruns
        numruns += 1

    messages = []
    conn = MockConnection(messages)
    c = context.Context(function)
    c.start(conn)

    old_numruns = numruns
    c.refresh()
    time.sleep(0.1)
    assert numruns > old_numruns

    c.stop()


def test_context_duplicate_key():
    numruns = 0

    def function():
        nonlocal numruns
        numruns += 1
        b = hd.button()
        # TODO: These assertions should happen on the main thread.
        assert_raises(Exception, hd.button, key=b.key)

    messages = []
    conn = MockConnection(messages)
    c = context.Context(function)
    c.start(conn)

    # Verify that it ran
    assert numruns > 0

    c.stop()


def test_context_current_context():
    numruns = 0

    def function():
        nonlocal numruns
        numruns += 1

        # TODO: These assertions should happen on the main thread:

        # A current context must be active
        assert context.Context.current() is not None

        # Can't set context while one is active
        assert_raises(Exception, context.Context.set_current_context, "foo")

    messages = []
    conn = MockConnection(messages)
    c = context.Context(function)
    c.start(conn)

    assert numruns > 0
    # Can't call current outside a hyperdiv function
    assert_raises(Exception, context.Context.current)

    c.stop()


def test_context_closed_connection():
    numruns = 0
    button = None
    clicked = False

    def function():
        nonlocal numruns, button, clicked
        numruns += 1

        button = hd.button()
        if button.clicked:
            clicked = True

    messages = []
    conn = MockConnection(messages)
    c = context.Context(function)
    c.start(conn)

    # Simulate a closed connection
    conn.opened.clear()
    # "Click the button again
    c.set_state(button.key, "clicked", True)
    # Some bogus updates that will hit code that logs warnings:
    c.set_state_bulk([(button.key, "foo", True), ("foobar", "foo", True)])
    time.sleep(0.1)
    # The connection re-opens
    conn.opened.set()
    time.sleep(0.1)
    assert clicked

    c.stop()


def test_context_timing_enabled():
    # This test just exercises code, it doesn't assert anything.
    def function():
        pass

    conn = MockConnection([])
    c = context.Context(function)
    c.timing_enabled = True
    c.start(conn)
    c.refresh()
    time.sleep(0.1)
    c.stop()

==> hyperdiv-0.0.13-py3-none-any.whl: hyperdiv/core/tests/context_tests.py <==
import time
import threading
from nose.tools import assert_raises
import hyperdiv as hd
from .. import context
from ..unittests import simple_test
from .utils import MockConnection


def test_event_bus():
    results = []

    def listener1(*args, **kwargs):
        results.append(("listener1", args, kwargs))

    def listener2(*args, **kwargs):
        results.append(("listener2", args, kwargs))

    bus = context.EventBus()

    bus.on("foo", listener1)
    bus.on("foo", listener2)
    bus.on("bar", listener1)

    assert bus.listeners == dict(foo=[listener1, listener2], bar=[listener1])

    bus.dispatch("foo", 1, x=2)
    bus.dispatch("bar", 2, 3, y=3, z=4)

    assert results == [
        ("listener1", (1,), {"x": 2}),
        ("listener2", (1,), {"x": 2}),
        ("listener1", (2, 3), {"y": 3, "z": 4}),
    ]

    bus.off("foo", listener1)

    assert bus.listeners == dict(foo=[listener2], bar=[listener1])

    bus.off("foo")

    assert bus.listeners == dict(bar=[listener1])

    bus.off("bar", listener1)

    assert bus.listeners == dict()


def test_collector_stack():
    s = context.CollectorStack()

    collector1 = []
    s.push(collector1)
    s.collect("Bunnies")
    s.collect("Bears")
    assert s.stack == [["Bunnies", "Bears"]] == [collector1]
    assert s.current() == collector1

    collector2 = []
    s.push(collector2)
    s.collect("Apples")
    s.collect("Oranges")
    assert (
        s.stack
        == [["Bunnies", "Bears"], ["Apples", "Oranges"]]
        == [collector1, collector2]
    )

    s.paused = True
    s.collect("Hello")
    assert s.stack == [collector1, collector2]
    s.paused = False

    assert s.pop() == collector2
    assert s.stack == [collector1]
    assert s.pop() == collector1
    assert s.stack == []

    assert_raises(Exception, s.current)


@simple_test
def test_paused_collector(ctx):
    with context.paused_collector():
        ctx.collector_stack.collect("Bunnies")
        ctx.collector_stack.collect("Bears")
    assert ctx.collector_stack.stack == [[]]


def test_scope_stack():
    s = context.ScopeStack()
    s.push("Bunnies")
    s.push("Bears")
    s.push("Apples")
    assert s.stack == ["Bunnies", "Bears", "Apples"]
    assert str(s) == "Bunnies:Bears:Apples"
    s.pop()
    assert s.stack == ["Bunnies", "Bears"]
    assert str(s) == "Bunnies:Bears"
    s.pop()
    s.pop()
    assert s.stack == []
    assert str(s) == ""


def test_state_update_batch():
    b = context.StateUpdateBatch()
    assert b.state == dict()

    b.set_state("a", "b", "value1")
    b.set_state("c", "d", "value2")

    assert b.has_state("a", "b")
    assert b.has_state("c", "d")
    assert not b.has_state("x", "y")

    assert_raises(Exception, b.set_state, "a", "b", "value")

    assert b.state == {("a", "b"): "value1", ("c", "d"): "value2"}


def test_pending_batches():
    evt = threading.Event()
    pb = context.PendingBatches(evt)
    pb.set_state_bulk(
        [
            ("a", "b", "value1"),
            ("c", "d", "value2"),
            ("x", "y", "value3"),
        ]
    )
    pb.set_state_bulk([("a", "b", "value4")])
    pb.set_state_bulk([("foo", "bar", "value5")])
    pb.set_state_bulk([("a", "b", "value6")])
    pb.set_state_bulk([("bam", "baz", "value7")])
    pb.set_state_bulk([("foo", "bar", "value8")])

    # Does nothing
    pb.set_state_bulk([])

    # Can't repeat keys in a batch
    assert_raises(
        Exception,
        pb.set_state_bulk,
        [("a", "b", "value9"), ("c", "d", "value10"), ("a", "b", "value11")],
    )

    assert len(pb.pending_batches) == 3
    [batch1, batch2, batch3] = pb.pending_batches

    assert batch1.state == {
        ("a", "b"): "value1",
        ("c", "d"): "value2",
        ("x", "y"): "value3",
        ("foo", "bar"): "value5",
        ("bam", "baz"): "value7",
    }
    assert batch2.state == {
        ("a", "b"): "value4",
        ("foo", "bar"): "value8",
    }
    assert batch3.state == {
        ("a", "b"): "value6",
    }

    assert pb.dequeue_batch() == batch1
    assert pb.dequeue_batch() == batch2
    assert pb.dequeue_batch() == batch3
    assert pb.dequeue_batch() is None

    assert pb.pending_batches == []


def test_context():
    button = None
    text = None

    def function():
        nonlocal button, text
        button = hd.button("Hello")
        text = hd.text("Hello")

    messages = []
    conn = MockConnection(messages)
    c = context.Context(function)
    context.Context.set_current_context(c)
    c.start(conn)

    conn.has_message.wait()

    assert len(messages) == 1
    msg = messages[0]

    assert "listeners" not in msg
    assert "children" in msg["dom"]  # root container

    # head and location singletons are first
    assert msg["dom"]["children"][0]["name"] == "theme"
    assert msg["dom"]["children"][1]["name"] == "head"
    assert msg["dom"]["children"][2]["name"] == "location"
    # our button and text are next
    assert msg["dom"]["children"][3] == button.to_dom()
    assert msg["dom"]["children"][4] == text.to_dom()
    context.Context.clear_current_context()
    c.stop()


def test_context_update():
    button = None
    text = None

    def function():
        nonlocal button, text
        button = hd.button("Hello")
        state = hd.state(shown=False)
        if state.shown:
            text = hd.text("Hello")
        if button.clicked:
            state.shown = True

    messages = []
    conn = MockConnection(messages)
    c = context.Context(function)
    context.Context.set_current_context(c)
    c.start(conn)

    conn.has_message.wait()

    assert len(messages) == 1
    msg = messages[0]

    assert msg["listeners"] == dict(add=[(button.key, "clicked")])
    assert "children" in msg["dom"]  # root container

    assert len(msg["dom"]["children"]) == 4
    assert msg["dom"]["children"][0]["name"] == "theme"
    assert msg["dom"]["children"][1]["name"] == "head"
    assert msg["dom"]["children"][2]["name"] == "location"
    # Text is not yet visible, only the button
    assert msg["dom"]["children"][3] == button.to_dom()

    # Simulate click:

    c.set_state(button.key, "clicked", True)
    # Sleep a bit to increase the chances that the update is applied.
    time.sleep(0.1)

    # One more message was sent:
    assert len(messages) == 2
    msg = messages[1]

    assert "diff" in msg
    assert "dom" not in msg

    # The message is a diff inserting the text element after the button
    assert msg["diff"] == [
        ("subdiff_list", "children", [("insert", 4, [text.to_dom()])])
    ]
    context.Context.clear_current_context()
    c.stop()


def test_context_refresh():
    numruns = 0

    def function():
        nonlocal numruns
        numruns += 1

    messages = []
    conn = MockConnection(messages)
    c = context.Context(function)
    c.start(conn)

    old_numruns = numruns
    c.refresh()
    time.sleep(0.1)
    assert numruns > old_numruns

    c.stop()


def test_context_duplicate_key():
    numruns = 0

    def function():
        nonlocal numruns
        numruns += 1
        b = hd.button()
        # TODO: These assertions should happen on the main thread.
        assert_raises(Exception, hd.button, key=b.key)

    messages = []
    conn = MockConnection(messages)
    c = context.Context(function)
    c.start(conn)

    # Verify that it ran
    assert numruns > 0

    c.stop()


def test_context_current_context():
    numruns = 0

    def function():
        nonlocal numruns
        numruns += 1

        # TODO: These assertions should happen on the main thread:

        # A current context must be active
        assert context.Context.current() is not None

        # Can't set context while one is active
        assert_raises(Exception, context.Context.set_current_context, "foo")

    messages = []
    conn = MockConnection(messages)
    c = context.Context(function)
    c.start(conn)

    assert numruns > 0
    # Can't call current outside a hyperdiv function
    assert_raises(Exception, context.Context.current)

    c.stop()


def test_context_closed_connection():
    numruns = 0
    button = None
    clicked = False

    def function():
        nonlocal numruns, button, clicked
        numruns += 1

        button = hd.button()
        if button.clicked:
            clicked = True

    messages = []
    conn = MockConnection(messages)
    c = context.Context(function)
    c.start(conn)

    # Simulate a closed connection
    conn.opened.clear()
    # "Click the button again
    c.set_state(button.key, "clicked", True)
    # Some bogus updates that will hit code that logs warnings:
    c.set_state_bulk([(button.key, "foo", True), ("foobar", "foo", True)])
    time.sleep(0.1)
    # The connection re-opens
    conn.opened.set()
    time.sleep(0.1)
    assert clicked

    c.stop()


def test_context_timing_enabled():
    # This test just exercises code, it doesn't assert anything.
    def function():
        pass

    conn = MockConnection([])
    c = context.Context(function)
    c.timing_enabled = True
    c.start(conn)
    c.refresh()
    time.sleep(0.1)
    c.stop()
